// Copyright © 2024 ESWIN. All rights reserved.
//
// Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
// all intellectual property and proprietary rights in and to this software. Except as expressly
// authorized by ESWIN, no part of the software may be released, copied, distributed, reproduced,
// modified, adapted, translated, or created derivative work of, in whole or in part.

#include <csignal>
#include <iostream>
#include <sstream>
#include <string>
#include <limits>
#include <thread>
#include <chrono>
#include <locale>
#include "es_qwen2_client.hpp"

#if BUILD_RUN_LLM == 1
#define USER_INPUT ": 自定义输入"
#else
#define USER_INPUT ": Customized prompts"
#endif

volatile std::sig_atomic_t signalReceived = 0;

void signalHandler(int signal)
{
    signalReceived = signal;
    if (signal == SIGINT) {
        std::cout << "\nQwen2 已退出" << std::endl;
        exit(signal);
    }
}

std::string getChineseInput()
{
    std::locale::global(std::locale(""));
    std::wstring winput;
    std::getline(std::wcin, winput);
    std::string input(winput.begin(), winput.end());

    for (char c : input) {
        if (!std::isalpha(c) || !std::isalpha(c, std::locale("zh_CN.UTF-8"))) {
            return "";
        }
    }

    return input;
}

int main(int argc, char** argv)
{
    std::string configFile = QWEN2_CONFIG_FILE;
    if (argc == 2) {
        configFile = std::string(argv[1]);
    }

    // Register the signal handler
    std::signal(SIGINT, signalHandler);

    const std::string splitStr = "----------------------------------------------------------------------------------";

    try {
        std::string inputPrompt;

        Qwen2Client Qwen2(configFile);

        while (!signalReceived) {
            inputPrompt.clear();

            std::map<int, std::string> options = Qwen2.getOptionalPrompts();
            if (options.size()) {
                std::cout << splitStr << std::endl;
                for (const auto& prompt : options) {
                    std::cout << prompt.first << ": " << prompt.second << std::endl;
                }
                std::cout << options.size() << USER_INPUT << std::endl;
                std::cout << splitStr << std::endl;
                std::cout << "[YOU]: ";

                int choice;
                std::string strChoice;
                std::getline(std::cin, strChoice);

                // 检查是否仅仅是回车
                if (strChoice.empty()) {
                    continue;
                }

                char extraChar;
                std::istringstream iss(strChoice);
                if (iss >> choice && !(iss >> extraChar)) {
                    auto it = options.find(choice);
                    if (it != options.end()) {
                        inputPrompt = it->second;
                        std::cout << "[YOU]: " << inputPrompt << std::endl;
                    } else {
                        if (choice == options.size()) {
                            std::cout << "[YOU]: ";
                            // inputPrompt = getChineseInput();
                            std::getline(std::cin, inputPrompt);
                            if (inputPrompt.empty()) {
                                continue;
                            }
                        } else {
                            continue;
                        }
                    }
                } else {
                    iss.clear();
                    continue;
                }
            } else {
                std::cout << "[YOU]: ";
                std::getline(std::cin, inputPrompt);
                if (inputPrompt.empty()) {
                    continue;
                }
            }

            // Generate response text
            std::cout << "[Qwen2]: ";
            std::string generatedText = Qwen2.genResponse(inputPrompt);
        }
    } catch (const std::exception& e) {
        return 1;
    }

    return 0;
}
